/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 *  Authors: Iain Holmes <iain@prettypeople.org>
 *
 *  Copyright 2002 Iain Holmes
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of version 2 of the GNU General Public 
 * License as published by the Free Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *
 */

#ifndef __MARLIN_BLOCK_H__
#define __MARLIN_BLOCK_H__

#include <glib.h>
#include <marlin-types.h>
#include <marlin-read-write-lock.h>

#define MARLIN_FRAMES_PER_PEAK 128
#define MARLIN_PEAKS_PER_BLOCK 4096

#define MARLIN_BLOCK_SIZE (MARLIN_PEAKS_PER_BLOCK * MARLIN_FRAMES_PER_PEAK)

struct _MarlinBlock {
	MarlinReadWriteLock *lock; /* Lock, do not access any data
				      without first obtaining the lock */
	
	struct _MarlinBlock *previous; /* A linked list of blocks */
	struct _MarlinBlock *next;

	gpointer frame_data; /* The actual data */
	
	guint64 start, end; /* The start and finish of this block */
	guint64 num_frames; /* The number of frames in frame_data */

	int scale_factor;
	int size; /* Size of a single entity in frame_data */
};

MarlinBlock *marlin_block_new_from_data (gpointer data,
					 guint64 num_frames,
					 int scale_factor,
					 int size);
void marlin_block_free (MarlinBlock *block);
void marlin_block_append (MarlinBlock *a,
			  MarlinBlock *b);
MarlinBlock *marlin_block_next (MarlinBlock *block);
MarlinBlock *marlin_block_previous (MarlinBlock *block);
MarlinBlock *marlin_block_split (MarlinBlock *block,
				 guint64 split_frame);
MarlinBlock *marlin_block_get_last (MarlinBlock *block);
#endif
