/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 *  Authors: Iain Holmes <iain@prettypeople.org>
 *
 *  Copyright 2002 Iain Holmes
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of version 2 of the GNU General Public 
 *  License as published by the Free Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *
 */

#ifndef __MARLIN_SAMPLE_SELECTION_H__
#define __MARLIN_SAMPLE_SELECTION_H__

#include <glib-object.h>

#include <marlin/marlin-sample.h>
#include <marlin/marlin-read-write-lock.h>

#define MARLIN_SAMPLE_SELECTION_TYPE (marlin_sample_selection_get_type ())
#define MARLIN_SAMPLE_SELECTION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), MARLIN_SAMPLE_SELECTION_TYPE, MarlinSampleSelection))
#define MARLIN_SAMPLE_SELECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), MARLIN_SAMPLE_SELECTION_TYPE, MarlinSampleSelectionClass))
#define IS_MARLIN_SAMPLE_SELECTION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MARLIN_SAMPLE_SELECTION_TYPE))
#define IS_MARLIN_SAMPLE_SELECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), MARLIN_SAMPLE_SELECTION_TYPE))

typedef struct _MarlinSampleSubSelection {
	MarlinCoverage coverage;

	guint64 initial; /* The place where the sample was initially started */
	guint64 start; /* Duh... */
	guint64 end; /* Double duh! */
} MarlinSampleSubSelection;

typedef struct _MarlinSampleSelection {
	GObject parent_object;
	
	MarlinReadWriteLock *lock; /* The lock, don't access anything
				      unless you hold the lock */
	
	MarlinSample *sample; /* Sample this selection refers to */
	GList *selections; /* Contains MarlinSampleSubSelection */
} MarlinSampleSelection;

typedef struct _MarlinSampleSelectionClass {
	GObjectClass parent_class;

	void (*changed) (MarlinSampleSelection *selection);
} MarlinSampleSelectionClass;

GType marlin_sample_selection_get_type (void);
MarlinSampleSelection *marlin_sample_selection_new (MarlinSample *sample);
MarlinSampleSubSelection *marlin_sample_selection_add (MarlinSampleSelection *selection,
						       guint64 start,
						       guint64 end,
						       MarlinCoverage coverage);
gboolean marlin_sample_selection_frame_in_selection (MarlinSampleSelection *selection,
						     MarlinSampleSubSelection **sub,
						     guint64 frame,
						     MarlinCoverage coverage);
void marlin_sample_selection_clear (MarlinSampleSelection *selection);

void marlin_sample_selection_changed (MarlinSampleSelection *selection,
				      gboolean compress);
#endif
			       
