/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*
 *  Authors: Iain Holmes <iain@prettypeople.org>
 *
 *  Copyright 2002 Iain Holmes
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
 *
 */

#ifndef __MARLIN_SAMPLE_H__
#define __MARLIN_SAMPLE_H__

#include <glib-object.h>
#include <gst/gst.h>

#include <marlin/marlin-object.h>
#include <marlin/marlin-types.h>
#include <marlin/marlin-operation.h>

#ifdef __cplusplus
extern "C" {
#pragma }
#endif

#define MARLIN_SAMPLE_TYPE (marlin_sample_get_type ())
#define MARLIN_SAMPLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), MARLIN_SAMPLE_TYPE, MarlinSample))
#define MARLIN_SAMPLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CASE ((klass), MARLIN_SAMPLE_TYPE, MarlinSampleClass))
#define IS_MARLIN_SAMPLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MARLIN_SAMPLE_TYPE))
#define IS_MARLIN_SAMPLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((obj), MARLIN_SAMPLE_TYPE))

typedef struct _MarlinSampleClass MarlinSampleClass;
typedef struct _MarlinSamplePrivate MarlinSamplePrivate;

typedef enum {
	MARLIN_COVERAGE_BOTH,
	MARLIN_COVERAGE_LEFT,
	MARLIN_COVERAGE_RIGHT
} MarlinCoverage;

typedef struct _MarlinSampleHeader {
	guint64 number_of_frames; /* Total number of frames */
	int channels; /* Number of channels */
	guint depth; /* Bit depth */
	guint sample_rate; /* Sample rate */
	float length; /* Length of sample in seconds */
} MarlinSampleHeader;

struct _MarlinSample {
	MarlinObject parent;

	MarlinSamplePrivate *priv;
};

struct _MarlinSampleClass {
	MarlinObjectClass parent_class;
};

GType marlin_sample_get_type (void);
MarlinSample *marlin_sample_new (void);
MarlinSample *marlin_sample_new_from_sample_with_range (MarlinSample *src,
							guint64 start_frame,
							guint64 finish_frame,
							MarlinCoverage coverage);
MarlinSample *marlin_sample_new_from_sample (MarlinSample *src);

gpointer marlin_sample_write_and_mmap_data (MarlinSample *sample,
					    float *data,
					    GError **error);

MarlinChannel *marlin_sample_get_channel (MarlinSample *sample,
					  guint channel_num);

GstElement *marlin_sample_get_filesrc (MarlinSample *sample);
void marlin_sample_generate_peaks (MarlinSample *sample,
				   MarlinOperation *operation,
				   GError **error);
void marlin_sample_ensure_mmapped (MarlinSample *sample,
				   MarlinOperation *operation,
				   GError **error);

/* Sample editting functions */
void marlin_sample_delete_range (MarlinSample *sample,
				 MarlinOperation *operation,
				 guint64 start,
				 guint64 finish,
				 GError **error);
void marlin_sample_clear_range (MarlinSample *sample,
				MarlinOperation *operation,
				guint64 start,
				guint64 finish,
				MarlinCoverage coverage,
				GError **error);
void marlin_sample_crop_range (MarlinSample *sample,
			       MarlinOperation *operation,
			       guint64 start,
			       guint64 finish,
			       GError **error);
void marlin_sample_swap_channels (MarlinSample *sample,
				  MarlinOperation *operation,
				  GError **error);
void marlin_sample_mix (MarlinSample *dest,
			MarlinSample *src,
			double src_db,
			double dest_db,
			guint64 start_frame,
			guint64 end_frame,
			gboolean clip,
			MarlinOperation *operation,
			GError **error);

void marlin_sample_insert (MarlinSample *dest,
			   MarlinSample *src,
			   guint64 position,
			   MarlinCoverage coverage);
void marlin_sample_invert_range (MarlinSample *sample,
				 guint64 start,
				 guint64 finish,
				 MarlinCoverage coverage);
void marlin_sample_adjust_volume_range (MarlinSample *sample,
					float db,
					guint64 start,
					guint64 finish,
					MarlinCoverage coverage);

void marlin_sample_insert_silence (MarlinSample *sample,
				   guint64 position,
				   guint64 length);
#ifdef __cplusplus
}
#endif

#endif
